<!DOCTYPE style-sheet PUBLIC "-//James Clark//DTD DSSSL Style Sheet//EN">
<style-sheet>
  <style-specification>
    <style-specification-body>

; jade_ex3.dsl: jade_ex2.dsl with nicer page footer and better
;               DSSSL programming practices.


; ------ constants ------

(define ListIndent   18pt)
(define BodyTextSize 12pt)
(define ParSpacing    8pt)
(define BulletChar   "\U-25CF")    ; Unicode of a solid circle


; ------ defined functions ------

(define (trim-lead-0 numstring)  ; trim leading zeros from numstring
        (cond ((string=? (substring numstring 0 1) "0")
         (trim-lead-0 (substring numstring 1 (string-length numstring))))
         (else numstring)
        )
)

(define date-str
; Pull "12/6/96" out of ISO format "1996-12-06T23:09:24"
  (let ((ISO-date (substring (time->string (time)) 0 10)))  ; e.g. "1996-12-06"
    (let ((month-num (substring ISO-date 5 7))
          (day-num (trim-lead-0 (substring ISO-date 8 10)))
          (year-num (substring ISO-date 2 4))
         )
         (string-append month-num "/ " day-num "/" year-num)
    )
  )
)

(define time-str 
; Pull "11:09 PM" out of ISO format "1996-12-06T23:09:24"
  (let ((ISO-hour (substring (time->string (time)) 11 13))    ; .e.g. "23"
        (ISO-minute (substring (time->string (time)) 14 16))) ; .e.g. "09"
    (let ((time-string 
          (cond ((string=? ISO-hour "12")
                 (string-append "12:" ISO-minute " PM"))
                ((string=? ISO-hour "00")
                 (string-append "12:" ISO-minute " AM"))
                ((> (string->number ISO-hour) 12)
                 (string-append 
                   (number->string (- (string->number ISO-hour) 12))
                   ":" ISO-minute " PM"
                 )
                )
                (else (string-append 
                        (trim-lead-0 ISO-hour) ":" ISO-minute " AM"
                      )
                )
          )     
         ))
      time-string  ; Return time string computed in the inner let statement.
    ) 
  )
)


; ------ styles defined for later use ------

(define footer-style
  (style
    font-size: 10pt
    font-posture: 'italic
  )
)

(define title-style
  (style
    font-family-name: "Arial"
    font-weight: 'bold
  )
)

(define listitem-par-style
  (style
    font-size: BodyTextSize
    start-indent: ListIndent
    first-line-start-indent: (- ListIndent) 
  )
)


; ------- construction rules ------

(element CHAPTER
  (let 
    ((page-num-footer 
       (make sequence
         use: footer-style
         (literal "Page ")
         (page-number-sosofo)
       )
     )
     (date-footer
       (make sequence
         use: footer-style
         (literal date-str)
       )
     )
     (time-footer
       (make sequence
         use: footer-style
         (literal time-str)
       )
     )
    )
    (make simple-page-sequence
      page-width:  8.5in
      page-height:  11in
      left-margin:   1in
      right-margin:  1in
      top-margin:    1in
      bottom-margin: 1in
      footer-margin: .5in
      left-footer: page-num-footer
      center-footer: date-footer
      right-footer: time-footer 
      (process-children)
    )
  ) 
)

(element SECT1
  (make display-group
    (process-children)
  )
)

(element SECT2
  (make display-group
    (process-children)
  )
)

(element SECT3
  (make display-group
    (process-children)
  )
)

(element (CHAPTER TITLE)
  (make paragraph
    use: title-style
    font-size: 24pt
    space-before: 20pt
    space-after: ParSpacing
    (process-children)
  )
)

(element (SECT1 TITLE)
  (make paragraph
    use: title-style
    font-size: 20pt
    space-before: 20pt
    space-after: ParSpacing
    (process-children)
  )
)

(element (SECT2 TITLE)
  (make paragraph
    use: title-style
    font-size: 16pt
    space-before: 16pt
    space-after: ParSpacing
    (process-children)
  )
)

(element (SECT3 TITLE)
  (make paragraph
    use: title-style
    font-posture: 'italic
    font-size: 12pt
    space-before: 12pt
    space-after: ParSpacing
    (process-children)
  )
)

(element ITEMIZEDLIST
  (make display-group
    space-after: ParSpacing
    (process-children)
  )
)

(element ORDEREDLIST
  (make display-group
    (process-children)
  )
)

(element (ITEMIZEDLIST LISTITEM) 
  (make display-group
    space-before: ParSpacing
    (process-children)
  )
)   

(element (ORDEREDLIST LISTITEM) 
  (make display-group
    space-before: ParSpacing
    (process-children)
  )
)   

(element (ITEMIZEDLIST LISTITEM PARA) 
  ; Thanks to Ken Holman for his help with this one.
  (make paragraph                   
    use: listitem-par-style
    space-after: 6pt
    (make line-field         
      field-width: ListIndent 
      (if (= (child-number) 1 ) ; Only need bullet in first para.
         (literal BulletChar) 
         (empty-sosofo)         ; Nothing if not first para of listitem.
      )
    )  
    (process-children)
  )
)  

(element (ORDEREDLIST LISTITEM PARA) 
  (make paragraph                   
    use: listitem-par-style
    space-after: 6pt
    (make line-field  
      field-width: ListIndent  
      (if (= (child-number) 1 ) ; Only need bullet in first para.
        (literal 
          (string-append 
            (format-number (ancestor-child-number "LISTITEM") "1") "."
          )
        )
        (empty-sosofo)          ; Nothing if not first para of item.
      )
    ) 
    (process-children)
  )
)  

(element LISTITEM
  (make display-group
    (process-children)
  )
)

(element PARA
  (make paragraph  
    font-family-name: "Times New Roman"
    font-size: BodyTextSize
    space-after: ParSpacing
    (process-children)
  )
)

(element LITERAL    ; Not "literal" in DSSSL sense, but an actual element type.
  (make sequence
    font-family-name: "Courier New"
    (process-children)
  )
)

(element EMPHASIS
  (make sequence
    font-posture: 'italic
    (process-children)
  )
)

(element GRAPHIC
  (make external-graphic
   entity-system-id: (attribute-string "fileref")
   notation-system-id: (attribute-string "format")
   display?: #t
   space-before: ParSpacing
   space-after: ParSpacing
  )
)

</style-specification-body>
  </style-specification>
</style-sheet>

